searchData={"items":[{"type":"module","title":"Kubereq","doc":"Kubereq defines a set of Request Steps for `Req`. All steps combined turn\na Kubernetes configuration in the form of a `%Kubereq.Kubeconfig{}` struct into a\n`%Req.Request{}` struct containing all headers and options required to\nconnect to the cluster and perform the given operations.\n\nIn order to build `%Kubereq.Kubeconfig{}` struct you can either use the steps defined\nin the `Kubeconf` library or create your own Kubernetes configuration loader\nmodule combining those steps.\n\nInstead of using this module directly, consider using\n[`Kubegen`](https://github.com/mruoss/kubegen) to generate your API clients.","ref":"Kubereq.html"},{"type":"module","title":"Examples - Kubereq","doc":"The following is a simple but incomplete example for a Client dealing\nwith ConfigMaps. This code is an extraction of what is generated by\n[`Kubegen`](https://github.com/mruoss/kubegen) when generating a client for\nConfigMaps.\n\n    defmodule MyApp.K8sClient.Core.V1.ConfigMap do\n      @resource_path \"api/v1/namespaces/:namespace/configmaps/:name\"\n\n      defp req() do\n        kubeconfig = Kubereq.Kubeconfig.load(Kubereq.Kubeconfig.Default)\n        Kubereq.new(kubeconfig, @resource_path)\n      end\n\n      def get(namespace, name) do\n        Kubereq.get(req(), namespace, name)\n      end\n\n      def list(namespace, opts \\\\ []) do\n        Kubereq.list(req(), namespace, opts)\n      end\n    end","ref":"Kubereq.html#module-examples"},{"type":"function","title":"Kubereq.apply/4","doc":"Applies the given `resource` using a Server-Side-Apply Patch.\nThe `req` struct should have been created using `Kubereq.new/2`.\n\nSee the [documentation](https://kubernetes.io/docs/reference/using-api/server-side-apply/)\nfor a documentation on `field_manager` and `force` arguments.","ref":"Kubereq.html#apply/4"},{"type":"function","title":"Examples - Kubereq.apply/4","doc":"iex> Kubereq.apply(req, %{...})\n    {:ok, %Req.Response{...}","ref":"Kubereq.html#apply/4-examples"},{"type":"function","title":"Kubereq.create/2","doc":"Create the `resource` object.\nThe `req` struct should have been created using `Kubereq.new/2`.","ref":"Kubereq.html#create/2"},{"type":"function","title":"Example - Kubereq.create/2","doc":"iex> Kubereq.create(req, resource)\n    {:ok, %Req.Response{status: 201, body: %{...}}}","ref":"Kubereq.html#create/2-example"},{"type":"function","title":"Kubereq.delete/3","doc":"Deletes a resource.\nThe `req` struct should have been created using `Kubereq.new/2`.","ref":"Kubereq.html#delete/3"},{"type":"function","title":"Examples - Kubereq.delete/3","doc":"iex> Kubereq.delete(req, \"default\", \"foo\")\n    {:ok, %Req.Response{status: 200, body: %{...}}}","ref":"Kubereq.html#delete/3-examples"},{"type":"function","title":"Kubereq.delete_all/3","doc":"Deletes all resources in the given namespace.\nThe `req` struct should have been created using `Kubereq.new/2`.","ref":"Kubereq.html#delete_all/3"},{"type":"function","title":"Examples - Kubereq.delete_all/3","doc":"iex> Kubereq.delete_all(req, \"default\", \"foo\")\n    {:ok, %Req.Response{...}","ref":"Kubereq.html#delete_all/3-examples"},{"type":"function","title":"Options - Kubereq.delete_all/3","doc":"* `:field_selectors` - A list of field selectors. See `Kubereq.Step.FieldSelector` for more infos.\n* `:label_selectors` - A list of field selectors. See `Kubereq.Step.LabelSelector` for more infos.","ref":"Kubereq.html#delete_all/3-options"},{"type":"function","title":"Kubereq.get/3","doc":"Get the resource `name` in `namespace`.\nThe `req` struct should have been created using `Kubereq.new/2`.","ref":"Kubereq.html#get/3"},{"type":"function","title":"Example - Kubereq.get/3","doc":"iex> Kubereq.get(req, \"default\", \"foo\")\n    {:ok, %Req.Response{status: 200, body: %{...}}}","ref":"Kubereq.html#get/3-example"},{"type":"function","title":"Kubereq.json_patch/4","doc":"Patches the resource `name`in `namespace` using the given `json_patch`.\nThe `req` struct should have been created using `Kubereq.new/2`.","ref":"Kubereq.html#json_patch/4"},{"type":"function","title":"Examples - Kubereq.json_patch/4","doc":"iex> Kubereq.json_patch(req, %{...}, \"default\", \"foo\")\n    {:ok, %Req.Response{...}","ref":"Kubereq.html#json_patch/4-examples"},{"type":"function","title":"Kubereq.list/3","doc":"Get a resource list.\nThe `req` struct should have been created using `Kubereq.new/2`.","ref":"Kubereq.html#list/3"},{"type":"function","title":"Examples - Kubereq.list/3","doc":"iex> Kubereq.list(req, \"api/v1/namespaces/:namespace/configmaps\", \"default\", [])\n    {:ok, %Req.Response{status: 200, body: %{...}}}","ref":"Kubereq.html#list/3-examples"},{"type":"function","title":"Options - Kubereq.list/3","doc":"* `:field_selectors` - A list of field selectors. See `Kubereq.Step.FieldSelector` for more infos.\n* `:label_selectors` - A list of field selectors. See `Kubereq.Step.LabelSelector` for more infos.","ref":"Kubereq.html#list/3-options"},{"type":"function","title":"Kubereq.merge_patch/4","doc":"Patches the resource `name`in `namespace` using the given `merge_patch`.\nThe `req` struct should have been created using `Kubereq.new/2`.","ref":"Kubereq.html#merge_patch/4"},{"type":"function","title":"Examples - Kubereq.merge_patch/4","doc":"iex> Kubereq.merge_patch(req, %{...}, \"default\", \"foo\")\n    {:ok, %Req.Response{...}","ref":"Kubereq.html#merge_patch/4-examples"},{"type":"function","title":"Kubereq.new/1","doc":"Prepares a `Req.Request` struct for making HTTP requests to a Kubernetes\ncluster. The `kubeconfig` is the Kubernetes configuration in the form of a\n`%Kubereq.Kubeconfig{}` struct and should contain all informations required to connect\nto the Kubernetes cluster.","ref":"Kubereq.html#new/1"},{"type":"function","title":"Examples - Kubereq.new/1","doc":"iex> kubeconfig = Kubereq.Kubeconfig.load(Kubereq.Kubeconfig.Default)\n    ...> Kubereq.new(kubeconfig)\n    %Request.Req{...}","ref":"Kubereq.html#new/1-examples"},{"type":"function","title":"Kubereq.new/2","doc":"Prepares a `Req.Request` struct for a specific resource on a specific\nKubernetes cluster. The `kubeconfig` is the Kubernetes configuration in the\nform of a `%Kubereq.Kubeconfig{}` struct and should contain all informations required to\nconnect to the Kubernetes cluster.\n\nThe parameter `resource_path` should be the path on which the Kubernetes API\nServer listens for requests for the targeted resource kind. It should\ncontain placeholders for `:namespace` and `:name`.\n\nThe `:namespace` and `:name` are provided through the `:path_params` option\nbuilt into `Req` when making the request.","ref":"Kubereq.html#new/2"},{"type":"function","title":"Examples - Kubereq.new/2","doc":"Prepare a `Req.Request` for ConfigMaps:\n\n    iex> kubeconfig = Kubereq.Kubeconfig.load(Kubereq.Kubeconfig.Default)\n    ...> Kubereq.new(kubeconfig, \"api/v1/namespaces/:namespace/configmaps/:name\")\n    %Request.Req{...}","ref":"Kubereq.html#new/2-examples"},{"type":"function","title":"Kubereq.update/2","doc":"Updates the given `resource`.\nThe `req` struct should have been created using `Kubereq.new/2`.","ref":"Kubereq.html#update/2"},{"type":"function","title":"Examples - Kubereq.update/2","doc":"iex> Kubereq.update(req, %{...})\n    {:ok, %Req.Response{...}","ref":"Kubereq.html#update/2-examples"},{"type":"function","title":"Kubereq.wait_until/5","doc":"GET a resource and wait until the given `callback` returns true or the given\n`timeout` (ms) has expired.\nThe `req` struct should have been created using `Kubereq.new/2`.","ref":"Kubereq.html#wait_until/5"},{"type":"function","title":"Options - Kubereq.wait_until/5","doc":"* `timeout` - Timeout in ms after function terminates with `{:error, :timeout}`","ref":"Kubereq.html#wait_until/5-options"},{"type":"function","title":"Kubereq.watch/3","doc":"Watch events of all resources in `namespace`.\nThe `req` struct should have been created using `Kubereq.new/2`.","ref":"Kubereq.html#watch/3"},{"type":"function","title":"Examples - Kubereq.watch/3","doc":"iex> Kubereq.watch(req, \"default\", [])\n    {:ok, #Function<60.48886818/2 in Stream.transform/3>}\n\nIn order to watch events in all namespaces, pass `nil` as namespace:\n\n    iex> Kubereq.watch(req, nil, [])\n    {:ok, #Function<60.48886818/2 in Stream.transform/3>}","ref":"Kubereq.html#watch/3-examples"},{"type":"function","title":"Options - Kubereq.watch/3","doc":"* `:resource_version` - If given, starts to stream from the given `resourceVersion` of the resource list. Otherwise starts streaming from HEAD.\n* `:stream_to` - If set to a `pid`, streams events to the given pid. If set to `{pid, ref}`, the messages are in the form `{ref, event}`.\n* `:field_selectors` - A list of field selectors. See `Kubereq.Step.FieldSelector` for more infos.\n* `:label_selectors` - A list of field selectors. See `Kubereq.Step.LabelSelector` for more infos.","ref":"Kubereq.html#watch/3-options"},{"type":"function","title":"Kubereq.watch_single/4","doc":"Watch events of a single resources `name`in `namespace`.\nThe `req` struct should have been created using `Kubereq.new/2`.","ref":"Kubereq.html#watch_single/4"},{"type":"function","title":"Examples - Kubereq.watch_single/4","doc":"iex> Kubereq.watch_single(req, \"default\", [])\n    {:ok, #Function<60.48886818/2 in Stream.transform/3>}\n\nIn order to watch events in all namespaces, pass `nil` as namespace:\n\n    iex> Kubereq.watch_single(req, nil, [])\n    {:ok, #Function<60.48886818/2 in Stream.transform/3>}","ref":"Kubereq.html#watch_single/4-examples"},{"type":"function","title":"Options - Kubereq.watch_single/4","doc":"* `:stream_to` - If set to a `pid`, streams events to the given pid. If set to `{pid, ref}`, the messages are in the form `{ref, event}`","ref":"Kubereq.html#watch_single/4-options"},{"type":"type","title":"Kubereq.namespace/0","doc":"","ref":"Kubereq.html#t:namespace/0"},{"type":"type","title":"Kubereq.response/0","doc":"","ref":"Kubereq.html#t:response/0"},{"type":"type","title":"Kubereq.wait_until_callback/0","doc":"","ref":"Kubereq.html#t:wait_until_callback/0"},{"type":"type","title":"Kubereq.wait_until_response/0","doc":"","ref":"Kubereq.html#t:wait_until_response/0"},{"type":"type","title":"Kubereq.watch_response/0","doc":"","ref":"Kubereq.html#t:watch_response/0"},{"type":"module","title":"Kubereq.Access","doc":"Helper module to access maps in lists.","ref":"Kubereq.Access.html"},{"type":"function","title":"Kubereq.Access.find/2","doc":"Returns a function that accesses the first element for which `fun` returns a truthy value.\n\nThe returned function is typically passed as an accessor to `Kernel.get_in/2`,\n`Kernel.get_and_update_in/3`, and friends.","ref":"Kubereq.Access.html#find/2"},{"type":"function","title":"Examples - Kubereq.Access.find/2","doc":"iex> list = [%{name: \"john\", salary: 10}, %{name: \"francine\", salary: 30}]\n    iex> get_in(list, [Kubereq.Access.find(&(&1.name == \"john\")), :salary])\n    10\n    iex> get_and_update_in(list, [Kubereq.Access.find(&(&1.name == \"john\")), :salary], fn prev ->\n    ...>   {prev, 15}\n    ...> end)\n    {10, [%{name: \"john\", salary: 15}, %{name: \"francine\", salary: 30}]}\n\n`find/1` can also be used to pop elements out of a list or\na key inside of a list:\n\n    iex> list = [%{name: \"john\", salary: 10}, %{name: \"francine\", salary: 30}]\n    iex> pop_in(list, [Kubereq.Access.find(&(&1.name == \"francine\"))])\n    {%{name: \"francine\", salary: 30}, [%{name: \"john\", salary: 10}]}\n    iex> pop_in(list, [Kubereq.Access.find(&(&1.name == \"francine\")), :name])\n    {\"francine\", [%{salary: 30}, %{name: \"john\", salary: 10}]}\n\nWhen no match is found, the given default is used. This can be used to\nspecify defaults and safely traverse missing items.\n\n    iex> list = [%{name: \"john\", salary: 10}, %{name: \"francine\", salary: 30}]\n    iex> get_in(list, [Kubereq.Access.find(&(&1.name == \"adam\"), %{name: \"adam\", salary: 50}), :salary])\n    50\n    iex> get_and_update_in(list, [Kubereq.Access.find(&(&1.name == \"adam\"), %{name: \"adam\"}), :salary], fn prev ->\n    ...>   {prev, 50}\n    ...> end)\n    {nil, [%{name: \"adam\", salary: 50}, %{name: \"john\", salary: 10}, %{name: \"francine\", salary: 30}]}\n\nWhen multiple items exist for which `fun` return a truthy value, the first one is accessed.\n\n    iex> list = [%{name: \"john\", salary: 10}, %{name: \"john\", salary: 30}]\n    iex> get_in(list, [Kubereq.Access.find(&(&1.name == \"john\")), :salary])\n    10\n\nAn error is raised if the accessed structure is not a list:\n\n    iex> get_in(%{}, [Kubereq.Access.find(&(&1.name == \"john\"))])\n    ** (RuntimeError) Kubereq.Access.find/1 expected a list, got: %{}","ref":"Kubereq.Access.html#find/2-examples"},{"type":"function","title":"Kubereq.Access.find!/1","doc":"Returns a function that accesses the first element for which `fun` returns a truthy value.\n\nThe returned function is typically passed as an accessor to `Kernel.get_in/2`,\n`Kernel.get_and_update_in/3`, and friends.\n\nSimilar to find/2, but the returned function raises if the no item is found for which `fun` returns a truthy value.","ref":"Kubereq.Access.html#find!/1"},{"type":"function","title":"Examples - Kubereq.Access.find!/1","doc":"iex> list = [%{name: \"john\", salary: 10}, %{name: \"francine\", salary: 30}]\n    iex> get_in(list, [Kubereq.Access.find!(&(&1.name == \"john\")), :salary])\n    10\n    iex> get_and_update_in(list, [Kubereq.Access.find!(&(&1.name == \"john\")), :salary], fn prev ->\n    ...>   {prev, 15}\n    ...> end)\n    {10, [%{name: \"john\", salary: 15}, %{name: \"francine\", salary: 30}]}\n\n`find/1` can also be used to pop elements out of a list or\na key inside of a list:\n\n    iex> list = [%{name: \"john\", salary: 10}, %{name: \"francine\", salary: 30}]\n    iex> pop_in(list, [Kubereq.Access.find!(&(&1.name == \"francine\"))])\n    {%{name: \"francine\", salary: 30}, [%{name: \"john\", salary: 10}]}\n    iex> pop_in(list, [Kubereq.Access.find!(&(&1.name == \"francine\")), :name])\n    {\"francine\", [%{salary: 30}, %{name: \"john\", salary: 10}]}\n    iex> get_in(list, [Kubereq.Access.find!(&(&1.name == \"adam\")), :salary])\n    ** (ArgumentError) There is no item in the list for which the given function returns a truthy value.\n\nWhen multiple items exist for which `fun` return a truthy value, the first one is accessed.\n\n    iex> list = [%{name: \"john\", salary: 10}, %{name: \"john\", salary: 30}]\n    iex> get_in(list, [Kubereq.Access.find!(&(&1.name == \"john\")), :salary])\n    10\n\nAn error is raised if the accessed structure is not a list:\n\n    iex> get_in(%{}, [Kubereq.Access.find!(&(&1.name == \"john\"))])\n    ** (RuntimeError) Kubereq.Access.find!/1 expected a list, got: %{}","ref":"Kubereq.Access.html#find!/1-examples"},{"type":"module","title":"Kubereq.Kubeconfig","doc":"This is the `Pluggable.Token` for the pipeline loading the Kubernetes config.","ref":"Kubereq.Kubeconfig.html"},{"type":"function","title":"Kubereq.Kubeconfig.load/1","doc":"Loads the Kubernetes config by running the given `pipeline`. Returns the\nresulting `%Kubereq.Kubeconfig{}`.\n\n`pipeline` can be passed in the form of `{pipeline_module, opts}` tuples,\na single `pipeline_module` or a list of either.","ref":"Kubereq.Kubeconfig.html#load/1"},{"type":"function","title":"Example - Kubereq.Kubeconfig.load/1","doc":"Single pipeline module without opts passed as module:\n\n    Kubereq.Kubeconfig.load(Kubereq.Kubeconfig.Default)\n\nSingle pipeline module with opts:\n\n    Kubereq.Kubeconfig.load({Kubereq.Kubeconfig.File, path: \"/path/to/kubeconfig\"})\n\nList of either:\n\n    Kubereq.Kubeconfig.load([\n      Kubereq.Kubeconfig.ENV,\n      {Kubereq.Kubeconfig.File, path: \".kube/config\", relative_to_home?: true},\n      Kubereq.Kubeconfig.ServiceAccount\n    ])","ref":"Kubereq.Kubeconfig.html#load/1-example"},{"type":"function","title":"Kubereq.Kubeconfig.new!/1","doc":"Creates a new `%Kubereq.Kubeconfig{}` struct with the given fields","ref":"Kubereq.Kubeconfig.html#new!/1"},{"type":"function","title":"Kubereq.Kubeconfig.set_current_context/2","doc":"Sets the current context. This function sets `current_cluster` and\n`current_user` in the given `Kubereq.Kubeconfig.t()`","ref":"Kubereq.Kubeconfig.html#set_current_context/2"},{"type":"type","title":"Kubereq.Kubeconfig.t/0","doc":"The `%Kubereq.Kubeconfig{}` struct holds information required to connect to\nKubernetes clusters\n\nFor descriptions of the fields, refer to the\n[kubeconfig.v1](https://kubernetes.io/docs/reference/config-api/kubeconfig.v1/) documentation.","ref":"Kubereq.Kubeconfig.html#t:t/0"},{"type":"module","title":"Kubereq.Kubeconfig.Default","doc":"Default pipeline of pluggable steps for loading the Kubeconfig. Tries to\nbuild the Kubeconfig from different default settings.\n\n1. Checks for the `KUBECONFIG` environment variable. If it is set and pointing\n   to a Kubeconfig file, that file is imported.\n1. Checks for `$HOME/.kube/config`.\n1. Checks if running inside a Cluster and tries to connect using the Service\n   Account Token.","ref":"Kubereq.Kubeconfig.Default.html"},{"type":"module","title":"Example - Kubereq.Kubeconfig.Default","doc":"Usage in a pipeline created with `Pluggable.StepBuilder`:\n\n    defmodule MyApp.KubeconfLoader do\n      use Pluggable.StepBuilder\n\n      step Kubereq.Kubeconfig.Default\n    end","ref":"Kubereq.Kubeconfig.Default.html#module-example"},{"type":"module","title":"Kubereq.Kubeconfig.ENV","doc":"Pluggable step that loads the Kubeconfig from a config file whose location is\ndefined by an ENV variable. Uses `Kubereq.Kubeconfig.File` under the hood.\n\n### Examples\n\n    step Kubereq.Kubeconfig.ENV\n\nBy default, this step assumes the name of the variable to be `KUBECONFIG`.\nThis can be customized through the `:env_var` option\n\n    step Kubereq.Kubeconfig.ENV, env_var: SPECIAL_KUBECONFIG\n\n### Options\n\n* `env_var` - (optional) The name of the environment variable. Defaults to\n  `KUBECONFIG`\n* `!` - (optional. And yes, that's a valid atom) Raise an exception if the\n  env var not found. Defaults to `false`.","ref":"Kubereq.Kubeconfig.ENV.html"},{"type":"module","title":"Kubereq.Kubeconfig.File","doc":"Pluggable step that load the Kubeconfig from a file.\n\n```\nstep Kubereq.Kubeconfig.File, path: \"path/to/kubeconfig-integration.yaml\"\n```\n\nPass `:relative_to_home?` to interprete the `:path` relative to the current\nuser's home directory at run time.\n\n```\nstep Kubereq.Kubeconfig.File, path: \".kube/config\", relative_to_home?: true\n```\n\nIf the config file defined by the `:path` option is not found on disk, by\ndefault, the step gracefully returns the kubeconfig that was passed as\nargument. If you want the step to raise an `ArgumentException` instead, you\ncan set the option `!: true`.\n\n### Options\n\n* `:path` - Path to the config file.\n* `:relative_to_home` - (optional) Interprete the `:path` as relative to the\n  user's home directory at runtime. Defaults to `false`.\n* `:!` - (optional. And yes, that's a valid atom) Raise an exception if the\n  config file is not found. Defaults to `false`.\n* `:context` - (optional) Sets the current context in case there's multiple\n  contexts defined in the config file. Defaults to what's defined in the\n  \"current-context\" field in the loaded config.","ref":"Kubereq.Kubeconfig.File.html"},{"type":"module","title":"Kubereq.Kubeconfig.ServiceAccount","doc":"Pluggable step that builds the Kubeconfig using a Service Account's token for\nauthentication.\n\nWhen running the app inside a Kubernetes cluster, make sure RBAC is configured\ncorrectly and use this step. It reads the service account's JWC token and\nbuild the Kubeconfig accordingly.","ref":"Kubereq.Kubeconfig.ServiceAccount.html"},{"type":"module","title":"Examples - Kubereq.Kubeconfig.ServiceAccount","doc":"step Kubereq.Kubeconfig.ServiceAccount\n\nIf your token is mounted at a different location than the default, pass its\nlocation as `:path_to_folder`.\n\n    step Kubereq.Kubeconfig.ServiceAccount, path_to_folder: \"path/to/folder/with/token\"","ref":"Kubereq.Kubeconfig.ServiceAccount.html#module-examples"},{"type":"module","title":"Options - Kubereq.Kubeconfig.ServiceAccount","doc":"* `path_to_folder` - (optional) Path to the folder where the `token`, `ca.crt`\n  and `namespace` files of the service account are located. Defaults to\n  `\"/var/run/secrets/kubernetes.io/serviceaccount\"`\n* `:!` - (optional. And yes, that's a valid atom) Raise an exception if the\n  config file is not found. Defaults to `false`.","ref":"Kubereq.Kubeconfig.ServiceAccount.html#module-options"},{"type":"module","title":"Kubereq.Kubeconfig.Stub","doc":"Req testing conveniences for `kubereq` requests.\n\nSince `kubereq` is using `Req` under the hood, we can use\n[`Req.Test`](https://hexdocs.pm/req/Req.Test.html) to run requests through\nmocks/stubs. Using this step as your Kubeconfig pipeline, you can set a stub\non the `Req.Request` configured by this step.\n\nIn your tests, you can then use `Req.Test` according to its documentation.","ref":"Kubereq.Kubeconfig.Stub.html"},{"type":"module","title":"Example - Kubereq.Kubeconfig.Stub","doc":"Imagine we're building an app with a pod client that lists pods on the\ncluster.\n\nWe start off by defining a module for loading the Kubernetes config:\n\n    defmodule MyApp.Kubeconfig do\n      @kubeconfig_pipeline Application.compile_env(:myapp, :kubeconfig_pipeline)\n\n      def load(), do: Kubereq.Kubeconfig.load(@kubeconfig_pipeline)\n    end\n\nWe then implement the pod client using the Kubeconfig loader to create a\n`Req.Request`:\n\n    defmodule MyApp.PodClient do\n      @resource_path \"api/v1/namespaces/:namespace/pods/:name\"\n\n      def list(namespace) do\n        req = Kubereq.new(MyApp.Kubeconfig.load(), @resource_path)\n\n        {:ok, resp} = Kubereq.list(req, namespace)\n        resp.body[\"items\"]\n      end\n    end\n\nWe configure the kubeconfig pipeline for production using\n`Kubereq.Kubeconfig.Default` as our pipeline to load the Kubeconfig:\n\n    # config/prod.exs\n    config :myapp, kubeconfig_pipeline: Kubereq.Kubeconfig.Default\n\nIn tests, instead of sending requests to the cluster, we make the request\nagainst a plug stub named `MyApp.Cluster`:\n\n    # config/test.exs\n    config :myapp, kubeconfig_pipeline: {Kubereq.Kubeconfig.Stub, plugs: {Req.Test, MyApp.Cluster}}\n\nNow we can control our stubs in concurrent tests:\n\n    use ExUnit.Case, async: true\n\n    test \"many pods\" do\n      Req.Test.stub(MyApp.Cluster, fn conn ->\n        Req.Test.json(conn, %{\n          \"apiVersion\" => \"v1\",\n          \"kind\" => \"List\",\n          \"items\" => [\n            %{\n              \"apiVersion\" => \"v1\",\n              \"kind\" => \"Pod\"\n              # ...\n            }\n          ]\n        })\n      end)\n\n      assert [_] = MyApp.PodClient.list(\"default\")\n    end","ref":"Kubereq.Kubeconfig.Stub.html#module-example"},{"type":"module","title":"Stubs per Context - Kubereq.Kubeconfig.Stub","doc":"If you want to simulate multiple different clusters or different responses for\nthe same calls, you can pass a `%{context :: String.t() => plug :: tuple}` map\nto as `plugs` option.\n\n    # config/test.exs\n    config :myapp, kubeconfig_pipeline: {Kubereq.Kubeconfig.Stub, plugs: %{\n      \"happy-path\" => {Req.Test, MyApp.HappyPathCluster}\n      \"missing-permissions\" => {Req.Test, MyApp.MissingPermissionsCluster}\n    }}\n\n## Options\n\n* `plugs` - The plug or `%{context => plug}` map to be configured on the\n  `Req.Request` configured by this step.","ref":"Kubereq.Kubeconfig.Stub.html#module-stubs-per-context"},{"type":"module","title":"Kubereq.Step.Auth","doc":"Req step to derive Req steps necessary for auth to the cluster.","ref":"Kubereq.Step.Auth.html"},{"type":"function","title":"Kubereq.Step.Auth.attach/1","doc":"","ref":"Kubereq.Step.Auth.html#attach/1"},{"type":"function","title":"Kubereq.Step.Auth.call/1","doc":"","ref":"Kubereq.Step.Auth.html#call/1"},{"type":"module","title":"Kubereq.Step.BaseUrl","doc":"Req step to derive the base URL to the cluster.","ref":"Kubereq.Step.BaseUrl.html"},{"type":"function","title":"Kubereq.Step.BaseUrl.attach/1","doc":"","ref":"Kubereq.Step.BaseUrl.html#attach/1"},{"type":"function","title":"Kubereq.Step.BaseUrl.call/1","doc":"","ref":"Kubereq.Step.BaseUrl.html#call/1"},{"type":"module","title":"Kubereq.Step.Compression","doc":"Req step to derive compression headers from the Kubeconfig.","ref":"Kubereq.Step.Compression.html"},{"type":"function","title":"Kubereq.Step.Compression.attach/1","doc":"","ref":"Kubereq.Step.Compression.html#attach/1"},{"type":"function","title":"Kubereq.Step.Compression.call/1","doc":"","ref":"Kubereq.Step.Compression.html#call/1"},{"type":"module","title":"Kubereq.Step.FieldSelector","doc":"Req step to format field selectors.\n\nField selectors let you select Kubernetes objects based on the value of one or more resource fields.\nThe concept is explained on the Kubernetes documentation\n[Field Selectors](https://kubernetes.io/docs/concepts/overview/working-with-objects/field-selectors/)\n\nThe functions for listing and watching resources accept an option\n`:field_selectors` as a list of strings or tuples.","ref":"Kubereq.Step.FieldSelector.html"},{"type":"module","title":"Examples - Kubereq.Step.FieldSelector","doc":"The following are equivalent field selectors for equality:\n\n* `\"metadata.name=my-service\"`\n* `\"metadata.name==my-service\"`\n* `{\"metadata.name\", \"my-service\"}\n* `{\"metadata.name\", {:eq, \"my-service\"}}\n\nThe following are equivalent field selectors for inequality:\n\n* `\"metadata.namespace!=default\"`\n* `{\"metadata.namespace\", {:neq, \"default\"}}`\n* `{\"metadata.namespace\", {:ne, \"default\"}}`","ref":"Kubereq.Step.FieldSelector.html#module-examples"},{"type":"function","title":"Kubereq.Step.FieldSelector.attach/1","doc":"","ref":"Kubereq.Step.FieldSelector.html#attach/1"},{"type":"function","title":"Kubereq.Step.FieldSelector.call/1","doc":"","ref":"Kubereq.Step.FieldSelector.html#call/1"},{"type":"module","title":"Kubereq.Step.Impersonate","doc":"Req step to derive impersonation headers from the Kubeconfig.","ref":"Kubereq.Step.Impersonate.html"},{"type":"function","title":"Kubereq.Step.Impersonate.attach/1","doc":"","ref":"Kubereq.Step.Impersonate.html#attach/1"},{"type":"function","title":"Kubereq.Step.Impersonate.call/1","doc":"","ref":"Kubereq.Step.Impersonate.html#call/1"},{"type":"module","title":"Kubereq.Step.LabelSelector","doc":"Req step to format label selectors.\n\nLabel selectors are used to filter list and watch operations by resource\nlabels. The concept is explained on the Kubernetes\ndocumentation [Labels and Selectors](https://kubernetes.io/docs/concepts/overview/working-with-objects/labels)\n\nThe functions for listing and watching resources accept an option\n`:label_selectors` as a list of strings or tuples.","ref":"Kubereq.Step.LabelSelector.html"},{"type":"module","title":"Equality-based requirements - Kubereq.Step.LabelSelector","doc":"The following are equivalent label selectors for equality:\n\n* `\"environment = production\"`\n* `{\"environment\", \"production\"}`\n* `{\"environment\", {:eq, \"production\"}}`\n\nThe following are equivalent label selectors for inequality:\n\n* `\"tier != frontend\"`\n* `{\"tier\", {:neq, \"frontend\"}}`\n* `{\"tier\", {:ne, \"frontend\"}}`","ref":"Kubereq.Step.LabelSelector.html#module-equality-based-requirements"},{"type":"module","title":"Set-based requirements - Kubereq.Step.LabelSelector","doc":"The following are equivalent label selectors for `In` requirements:\n\n* `\"environment in (production, qa)\"`\n* `{\"environment\", [\"production\", \"qa\"]}`\n* `{\"environment\", {:in, [\"production\", \"qa\"]}}`\n\nThe following are equivalent label selectors for `NotIn` requirements:\n\n* `\"tier notin (frontend, backend)\"`\n* `{\"tier\", {:notin, [\"frontend\", \"backend\"]}}`\n* `{\"tier\", {:not_in, [\"frontend\", \"backend\"]}}`\n\nThe following are equivalent label selectors for `Exists` requirements:\n\n* `\"partition\"`\n* `{\"partition\"}`\n* `{\"partition\", :exists}`\n\nThe following are equivalent label selectors for `not DoesNotExist` requirements:\n\n* `\"!partition\"`\n* `{\"!partition\"}`\n* `{\"partition\", :notexists}`\n* `{\"partition\", :not_exists}`","ref":"Kubereq.Step.LabelSelector.html#module-set-based-requirements"},{"type":"function","title":"Kubereq.Step.LabelSelector.attach/1","doc":"","ref":"Kubereq.Step.LabelSelector.html#attach/1"},{"type":"function","title":"Kubereq.Step.LabelSelector.call/1","doc":"","ref":"Kubereq.Step.LabelSelector.html#call/1"},{"type":"module","title":"Kubereq.Step.Plug","doc":"Req step to derive the base URL to the cluster.","ref":"Kubereq.Step.Plug.html"},{"type":"function","title":"Kubereq.Step.Plug.attach/1","doc":"","ref":"Kubereq.Step.Plug.html#attach/1"},{"type":"function","title":"Kubereq.Step.Plug.call/1","doc":"","ref":"Kubereq.Step.Plug.html#call/1"},{"type":"module","title":"Kubereq.Step.TLS","doc":"Req step to derive TLS configuration from the Kubeconfig.","ref":"Kubereq.Step.TLS.html"},{"type":"function","title":"Kubereq.Step.TLS.attach/1","doc":"","ref":"Kubereq.Step.TLS.html#attach/1"},{"type":"function","title":"Kubereq.Step.TLS.call/1","doc":"","ref":"Kubereq.Step.TLS.html#call/1"},{"type":"exception","title":"Kubereq.Error.KubeconfError","doc":"Indicates an error with the Kubernetes Configuration","ref":"Kubereq.Error.KubeconfError.html"},{"type":"function","title":"Kubereq.Error.KubeconfError.new/2","doc":"","ref":"Kubereq.Error.KubeconfError.html#new/2"},{"type":"type","title":"Kubereq.Error.KubeconfError.t/0","doc":"","ref":"Kubereq.Error.KubeconfError.html#t:t/0"},{"type":"exception","title":"Kubereq.Error.StepError","doc":"","ref":"Kubereq.Error.StepError.html"},{"type":"function","title":"Kubereq.Error.StepError.new/2","doc":"","ref":"Kubereq.Error.StepError.html#new/2"},{"type":"type","title":"Kubereq.Error.StepError.t/0","doc":"","ref":"Kubereq.Error.StepError.html#t:t/0"},{"type":"extras","title":"Kubereq","doc":"# Kubereq\n\nUsed by [`kubegen`](https://github.com/mruoss/kubegen) to build Resource based\nKubernetes API clients using Req with `kubereq`.\n\n[![Module Version](https://img.shields.io/hexpm/v/kubereq.svg)](https://hex.pm/packages/kubereq)\n[![Last Updated](https://img.shields.io/github/last-commit/mruoss/kubereq.svg)](https://github.com/mruoss/kubereq/commits/main)\n\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/kubereq/)\n[![Total Download](https://img.shields.io/hexpm/dt/kubereq.svg)](https://hex.pm/packages/kubereq)\n[![License](https://img.shields.io/hexpm/l/kubereq.svg)](https://github.com/mruoss/kubereq/blob/main/LICENSE.md)\n\nWhile this library can be used directly, it is easier to let\n[`kubegen`](https://github.com/mruoss/kubegen) generate the API client modules\nfor you. The resulting clients are then using `kubereq` to get the prepared\n`Req.Request` struct and make the requests to the Kubernetes API Server.","ref":"readme.html"},{"type":"extras","title":"Installation - Kubereq","doc":"The package can be installed by adding `kubereq` to your list of dependencies in\n`mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:kubereq, \"~> 0.2.0\"}\n  ]\nend\n```\n\nThe docs can be found at  .","ref":"readme.html#installation"},{"type":"extras","title":"Usage with [`kubegen`](https://github.com/mruoss/kubegen) - Kubereq","doc":"Unless you want to build your clients yourself, you can use\n[`kubegen`](https://github.com/mruoss/kubegen) to generate clients for each\nresource kind you need. Check out [`kubegen`](https://github.com/mruoss/kubegen).","ref":"readme.html#usage-with-kubegen-https-github-com-mruoss-kubegen"},{"type":"extras","title":"Build your own clients - Kubereq","doc":"","ref":"readme.html#build-your-own-clients"},{"type":"extras","title":"Define how to load the Kubernetes Config - Kubereq","doc":"In order to get started quickly, you can just use the default pipeline\n(`Kubereq.Kubeconfig.Default`) which tries to load the Kubernetes configuration\none-by-one from well-known sources.\n\nIf you need more sophisticated rules, you can build your own Kubeconfig loader\npipeline by creating a module `use`-ing [`Pluggable.StepBuilder`](https://hexdocs.pm/pluggable/Pluggable.StepBuilder.html)\nand adding `Pluggable` steps defined by this module. The mechanism is exactly\nthe same as you know from the `Plug` library.\n\nIn fact, the default pipeline mentioned above is implemented defining a set of\nsteps.\n\n```ex\ndefmodule Kubereq.Kubeconfig.Default do\n  use Pluggable.StepBuilder\n\n  step Kubereq.Kubeconfig.ENV\n  step Kubereq.Kubeconfig.File, path: \".kube/config\", relative_to_home?: true\n  step Kubereq.Kubeconfig.ServiceAccount\nend\n```","ref":"readme.html#define-how-to-load-the-kubernetes-config"},{"type":"extras","title":"Load the Kubernetes Config - Kubereq","doc":"With the pipeline defined or implemented, you can now call\n`Kubereq.Kubeconfig.load/1` to load the config:\n\n```ex\nKubereq.Kubeconfig.load(Kubereq.Kubeconfig.Default)\n```\n\nIf your pipelines requires options, you can pass a tuple to\n`Kubereq.Kubeconfig.load/1`:\n\n```ex\nKubereq.Kubeconfig.load({Kubereq.Kubeconfig.File, path: \".kube/config\", relative_to_home?: true})\n```\n\nInstead of creating a new module, you can also pass a list of steps to\n`Kubereq.Kubeconfig.load/1`:\n\n```ex\nKubereq.Kubeconfig.load([\n  Kubereq.Kubeconfig.ENV,\n  {Kubereq.Kubeconfig.File, path: \".kube/config\", relative_to_home?: true},\n  Kubereq.Kubeconfig.ServiceAccount\n])\n```","ref":"readme.html#load-the-kubernetes-config"},{"type":"extras","title":"Building the `Req.Request` struct - Kubereq","doc":"Once you have loaded the, you can pass it to `Kubereq.new/2` to get a\n`%Req.Request{}` struct which is prepared to make requests to the Kubernetes\nAPI Server for **a specific resource kind**. It expects the `kubeconf` as first\nargument and the `path` to the resource as second argument. The path should\ncontain placeholders for `:namespace` and `:name` which are filled once you make\na request to a specific resource.\n\nThe following example builds a `%Req.Request{}` which acts as client for running\noperations on `ConfigMaps`:\n\n```ex\nkubeconfig = Kubereq.Kubeconfig.load(Kubereq.Kubeconfig.Default)\nreq = Kubereq.new(kubeconfig, \"api/v1/namespaces/:namespace/configmaps/:name\")\n```","ref":"readme.html#building-the-req-request-struct"},{"type":"extras","title":"Running Operations - Kubereq","doc":"With the `req` built above, you can now use the other functions defined by\n`Kubereq` to run operations - in this example on `ConfigMaps`.\n\n```ex\nkubeconfig = Kubereq.Kubeconfig.load(Kubereq.Kubeconfig.Default)\nreq = Kubereq.new(kubeconfig, \"api/v1/namespaces/:namespace/configmaps/:name\")\n\n{:ok, resp} = Kubereq.get(req, \"my-namespace\", \"my-config-map\")\n```\n\n`resp` is a `Req.Response.t()` and you can check for `req.status` and get\n`req.body` etc.","ref":"readme.html#running-operations"},{"type":"extras","title":"Testing / Stubbing - Kubereq","doc":"Since `kubereq` is using `Req` under the hood, we can use\n[`Req.Test`](https://hexdocs.pm/req/Req.Test.html) to run requests through\nmocks/stubs. Use the special `Kubereq.Kubeconfig.Stub` to set stubs on the\nresulting `Req.Request` structs. See `Kubereq.Kubeconfig.Stub` for further\ndocumentation","ref":"readme.html#testing-stubbing"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"<!-- Add your changelog entry to the relevant subsection -->\n\n<!--","ref":"changelog.html#unreleased"},{"type":"extras","title":"Added | Changed | Deprecated | Removed | Fixed | Security --> - Changelog","doc":"<!--------------------- Don't add new entries after this line --------------------->","ref":"changelog.html#added-changed-deprecated-removed-fixed-security"},{"type":"extras","title":"0.2.0 - 2024-09-20 - Changelog","doc":"","ref":"changelog.html#0-2-0-2024-09-20"},{"type":"extras","title":"Changed - Changelog","doc":"- `exec` auth and other steps now return errors instead of raising exceptions. [#30](https://github.com/mruoss/kubereq/pull/30)","ref":"changelog.html#changed"},{"type":"extras","title":"0.1.8 - 2024-09-14 - Changelog","doc":"","ref":"changelog.html#0-1-8-2024-09-14"},{"type":"extras","title":"Changed - Changelog","doc":"- `Kubereq.Kubeconfig.Stub`: Set server url to context name","ref":"changelog.html#changed"},{"type":"extras","title":"0.1.7 - 2024-09-12 - Changelog","doc":"","ref":"changelog.html#0-1-7-2024-09-12"},{"type":"extras","title":"Added - Changelog","doc":"- `Kubereq.Kubeconfig.Stub`: A Kubeconfig step used for testing","ref":"changelog.html#added"},{"type":"extras","title":"0.1.6 - 2024-08-25 - Changelog","doc":"","ref":"changelog.html#0-1-6-2024-08-25"},{"type":"extras","title":"Fixed - Changelog","doc":"- `Kubereq.Kubeconfig.ServiceAccount`: make server key a binary","ref":"changelog.html#fixed"},{"type":"extras","title":"0.1.5 - 2024-08-24 - Changelog","doc":"","ref":"changelog.html#0-1-5-2024-08-24"},{"type":"extras","title":"Added - Changelog","doc":"- `Kubereq.wait_until/5`: Support returning `{:error, error}` tuple in the callback","ref":"changelog.html#added"},{"type":"extras","title":"0.1.4 - 2024-08-21 - Changelog","doc":"","ref":"changelog.html#0-1-4-2024-08-21"},{"type":"extras","title":"Fixed - Changelog","doc":"- `Kubereq.Kubeconfig.File`: Only expand path if not relative to HOME.","ref":"changelog.html#fixed"},{"type":"extras","title":"0.1.3 - 2024-08-19 - Changelog","doc":"","ref":"changelog.html#0-1-3-2024-08-19"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed several bugs and docs\n- Dependency updates","ref":"changelog.html#fixed"},{"type":"extras","title":"0.1.2 - 2024-06-13 - Changelog","doc":"- Dependency updates","ref":"changelog.html#0-1-2-2024-06-13"},{"type":"extras","title":"0.1.1 - Changelog","doc":"","ref":"changelog.html#0-1-1"},{"type":"extras","title":"Fixed - Changelog","doc":"- License was wrong in `mix.exs`","ref":"changelog.html#fixed"},{"type":"extras","title":"Unreleased - Changelog","doc":"<!-- Add your changelog entry to the relevant subsection -->\n\n<!--","ref":"changelog.html#unreleased"},{"type":"extras","title":"Added | Changed | Deprecated | Removed | Fixed | Security --> - Changelog","doc":"<!-- No new entries below this line! -->","ref":"changelog.html#added-changed-deprecated-removed-fixed-security"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}